% Least-Squares Luma-Chroma Demultiplexing Algorithm for Demosaicking
% IEEE Trans. Image Processing (submitted)
% This software is for provided for non-commercial and research purposes only
% Copyright Eric Dubois (edubois@site.uottawa.ca), University of Ottawa 2010 
%
%      [CFA] = create_CFA(RGB)
%      RGB(:,:,3)   input RGB image
%      CFA(:,:)     output Bayer CFA mosaicked image
%
%      Upper left pixel is green, pixel to right of it is red, pixel below it is
%      blue
%
%      G R
%      B G
%
function [CFA] = create_CFA(RGB,scheme)

% Check function arguments
error(nargchk(1, 2, nargin, 'struct'));
error(nargoutchk(1, 1, nargout, 'struct'));

if (nargin == 1)
    scheme = 1;
end

S = size(RGB); N1 = S(1); N2 = S(2);
CFA = zeros(N1,N2);

%Write four cosets of RGB to CFA
if scheme == 4
    CFA(1:2:N1,2:2:N2) = RGB(1:2:N1,2:2:N2,3); %blue
    CFA(1:2:N1,1:2:N2) = RGB(1:2:N1,1:2:N2,2); %green
    CFA(2:2:N1,2:2:N2) = RGB(2:2:N1,2:2:N2,2); %green
    CFA(2:2:N1,1:2:N2) = RGB(2:2:N1,1:2:N2,1); %red
elseif scheme == 3
    CFA(1:2:N1,2:2:N2) = RGB(1:2:N1,2:2:N2,2); %green
    CFA(1:2:N1,1:2:N2) = RGB(1:2:N1,1:2:N2,3); %blue
    CFA(2:2:N1,2:2:N2) = RGB(2:2:N1,2:2:N2,1); %red
    CFA(2:2:N1,1:2:N2) = RGB(2:2:N1,1:2:N2,2); %green
elseif scheme == 2
    CFA(1:2:N1,2:2:N2) = RGB(1:2:N1,2:2:N2,2); %green
    CFA(1:2:N1,1:2:N2) = RGB(1:2:N1,1:2:N2,1); %red
    CFA(2:2:N1,2:2:N2) = RGB(2:2:N1,2:2:N2,3); %blue
    CFA(2:2:N1,1:2:N2) = RGB(2:2:N1,1:2:N2,2); %green
else
    CFA(1:2:N1,2:2:N2) = RGB(1:2:N1,2:2:N2,1); %red
    CFA(1:2:N1,1:2:N2) = RGB(1:2:N1,1:2:N2,2); %green
    CFA(2:2:N1,2:2:N2) = RGB(2:2:N1,2:2:N2,2); %green
    CFA(2:2:N1,1:2:N2) = RGB(2:2:N1,1:2:N2,3); %blue
end